function y=fix_step_glitch(x,n)
% y=fix_step_glitch(x) - fix step-like glitches due to squid flux capture (?)
% 
%   x: data to fix (time * channels * trials)
%
%   y: fixed data
%
% Copyright 2008 Alain de Cheveigne


[m,n,o]=size(x);
x=unfold(x);

% perform a 2-cluster k-means clustering to get estimates of the means of
% stable parts

% initialize to min, max
c1=max(x);
c2=min(x);

% partition samples, update means (1st iteration)
mask=ones(size(x));
mask(find(abs(vecadd(x,-c1)) < abs(vecadd(x,-c2))))=0;
c1=sum(mask.*x)./sum(mask);
mask=1-mask;
c2=sum(mask.*x)./sum(mask);

% partition samples, update means (2nd iteration)
mask=ones(size(x));
mask(find(abs(vecadd(x,-c1)) < abs(vecadd(x,-c2))))=0;
c1=sum(mask.*x)./sum(mask);
mask=1-mask;
c2=sum(mask.*x)./sum(mask);

% variance from mean
v1=sum(vecadd(x,-mean(x)).^2);

% variance from nearest cluster
v2=sum(vecadd(x,-c2).^2.*mask) + sum(vecadd(x,-c1).^2.*(1-mask));

% decide whether we have a step-like glitch
criterion1 = 0.05;  % minimum relative duration of a glitch
criterion2 = 100; % minimum variance reduction ratio when variance is counted re clusters instead of re mean;
glitch_duration=min(sum(mask), sum(1-mask))/size(mask,1);

idx=find( (glitch_duration>criterion1) & (v1./v2 > criterion2));

% fix channels that need fixing
for k=idx;
    disp(['WARNING: removing a step-like glitch on channel ',num2str(k)]);
    subplot 121; plot(x(:,k)); title('before');
    x(:,k) = (x(:,k)-c2(k)).*mask(:,k) + (x(:,k)-c1(k)).*(1-mask(:,k));
    subplot 122; plot(x(:,k)); title('after');
    drawnow;
end
y=x;

y=fold(y,m);

